/*
 *  DotFrame.java
 *	Part of the Spirograph problem set.
 *
 * Developed for "Rethinking CS101", a project of Lynn Andrea Stein's AP Group.
 * For more information, see <a href="http://www.ai.mit.edu/projects/cs101">the
 * CS101 homepage</a> or email <las@ai.mit.edu>.
 *
 * Copyright (C) 1998 Massachusetts Institute of Technology.
 * Please do not redistribute without obtaining permission.
 */

package Spirograph;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

/** This class pops up the window for the Spirograph, creates all of
 * the buttons and the DotPanel, and adds them. It also creates an
 * instance of AdvEnv and passes it a listener that will set the
 * velocity of the dot. <p>
 *
 * Copyright (C) 1998 Massachusetts Institute of Technology.
 *
 * @author  Luis Sarmenta, lfgs@cag.lcs.mit.edu
 * @author  Henry Wong, henryw@mit.edu
 * @version  $Id: DotFrame.java,v 1.6 1998/07/24 16:58:42 tparnell Exp $
 * 
 * @see Coord
 * @see DotPanel
 * @see AccelHandler
 * @see AdvEnv
 * @see SetVListener
 * @see Spirograph
 */
public class DotFrame extends Frame {

  // Must be fields for anonymouse classes
  private Thread xSpirit, ySpirit, painter;
  private boolean xStarted = false, yStarted = false, pStarted = false;
  private DotPanel myPanel;
  private Coord x, y;

  // These fields are used to calculate the difference in size
  // between the Panel and the Frame. They are used in order to resize
  // the Panel after the Frame has been resized.
    
  private int xDif;
  private int yDif;

  private AdvEnv myAdvEnv;
    
  /**
   *  Takes care of GUI stuff. It also starts and suspends the three
   *  Threads (one for each of the AccelHandlers and one for the
   *  DotPanel).
   */
  public DotFrame(AccelHandler xHandler, AccelHandler yHandler,
		  DotPanel thePanel, Coord xCoord, Coord yCoord) {
    this(xHandler, yHandler, thePanel, xCoord, yCoord,
	 Spirograph.DEFAULTFONT);
  }
	
  /** Same as other constructor, except it sets the font rather than
   * using a default. */
  public DotFrame(AccelHandler xHandler, AccelHandler yHandler,
		  DotPanel thePanel, Coord xCoord, Coord yCoord, Font f) {
	
    this.setFont(f);
    this.setTitle("Spirograph");
	
    myPanel = thePanel;
    x = xCoord;
    y = yCoord;
		
    this.setLayout( new BorderLayout(10, 10)); 

    // Create the buttons and add some functionality.
    Button start=new Button("Start");
    start.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	if(xStarted) {
	  xSpirit.resume();
	} else {
	  xSpirit.start();
	  xStarted = true;
	}
	if(yStarted) {
	  ySpirit.resume();
	} else {
	  ySpirit.start();
	  yStarted = true;
	}
	if(pStarted) {
	  painter.resume();
	} else {
	  painter.start();
	  pStarted = true;
	}
      }
    });
	    
    Button stop = new Button("Stop");
    stop.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ae) {
	xSpirit.suspend();
	ySpirit.suspend();
	painter.suspend();
      }
    });

    // Create the ResetMenu and some listeners for its various
    // buttons
    final ResetMenu myRM = new ResetMenu();
    myRM.addListeners(new ActionListener() {
      public void actionPerformed(ActionEvent ae) {
	myPanel.flushLines();
	myPanel.paintBuf();
      }
    },
      new ActionListener() {
	public void actionPerformed(ActionEvent ae) {
	  myPanel.flushGrav();
	  myPanel.paintBuf();
	}
      },
        new ActionListener() {
	  public void actionPerformed(ActionEvent ae) {
	    xSpirit.suspend();
	    ySpirit.suspend();
	    painter.suspend();
	    x.setPos(0);
	    x.setVel(0);
	    y.setPos(0);
	    y.setVel(0);
	    myPanel.paintBuf();
	  }
	},
	  new ActionListener() {
	    public void actionPerformed(ActionEvent ae) {
	      xSpirit.suspend();
	      ySpirit.suspend();
	      painter.suspend();
	      x.setPos(0);
	      x.setVel(0);
	      y.setPos(0);
	      y.setVel(0);
	      myPanel.flushLines();
	      myPanel.flushGrav();
	      myPanel.paintBuf();
	    }
	  });
	    
    Button reset=new Button("Reset");
    reset.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	myRM.setVisible(true);
      }
    });


    // Create the textfields and listeners for AdvEnv and pass them in.
    TextField xVel = new TextField(3);
    TextField yVel = new TextField(3);
	
    SetPosGravListener panelListener = new SetPosGravListener(x,y,myPanel);
    myPanel.addMouseListener(panelListener);
    final AdvEnv myAdvEnv = new AdvEnv(xVel, yVel, panelListener, myPanel);
	
    // Add Listeners
    SetVListener myVelSetter = new SetVListener(xVel, yVel, x, y);
    myAdvEnv.addListener(myVelSetter);

    Button advEnv = new Button("Advanced Environment Options");
    advEnv.addActionListener(new ActionListener() {

      public void actionPerformed(ActionEvent ae) {
	myAdvEnv.setVisible(true);
      }
    });

    // Create the Quit Button
    Button quit=new Button("Quit");
    quit.addActionListener (new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	xSpirit.stop();
	ySpirit.stop();
	painter.stop();
	setVisible(false);
	dispose();
	System.exit(0);
      }
    });
	
    System.out.println ("Buttons created.");

    // Create the Button panel
    Panel p = new Panel();
    p.setLayout(new FlowLayout(FlowLayout.CENTER));
	
    p.add(start); 
    p.add(stop);
    p.add(reset);
    p.add(advEnv);
    p.add(quit);
	
    System.out.println ("Button panel created");
	    
    xSpirit = new Thread (xHandler);
    ySpirit = new Thread (yHandler);
    painter = new Thread (myPanel);

    this.add("Center", myPanel);
    this.add("South",p);
	
    // The Spirograph handles Resizing the window by changing the
    // size of the Panel. The variables xDif and yDif represents
    // the difference in size between the Frame and the Panel.
	    
    this.addComponentListener(new ComponentListener() {
      // if the window is resized, notify the appropriate
      // Objects.
		
      public void componentResized(ComponentEvent e) {
	int width = getSize().width-xDif;
	int height = getSize().height-yDif;
		    
	System.out.println ("Resized to " + width
			    + " wide and " +
			    height + " high.");
		
	myPanel.setSize(width,height);
      }

      // I need to implement these methods for the
      // ComponentListener interface.
      public void componentHidden(ComponentEvent e){}
      public void componentShown(ComponentEvent e){}
      public void componentMoved(ComponentEvent e){}
	    
    });
	    
    System.out.println ("Stuff added.");
	    
    this.pack();	
    this.setVisible(true);

    // xDif and yDif have to be set after Frame is made Visible,
    // otherwise the calls to getSize will return 0
	
    xDif = this.getSize().width - myPanel.getSize().width;
    System.out.println ("Frame width: " + xDif);
    yDif = this.getSize().height - myPanel.getSize().height;
    System.out.println ("Frame height: " + yDif);
	
    System.out.println ("Done.");
  }
}

/*
 * $Log: DotFrame.java,v $
 * Revision 1.6  1998/07/24 16:58:42  tparnell
 * Placate new javadoc behavior
 *
 * Revision 1.5  1998/07/24 15:51:34  tparnell
 * Added javadoc.  Removed some deprecated code.  Fixed a few minor bugs.
 *
 * Revision 1.4  1998/07/23 15:09:28  natashao
 * Added javadoc and a few comments.
 *
 */



