/*
 * DotPanel.java 
 * Part of the Spirograph problem set
 *
 * Developed for "Rethinking CS101", a project of Lynn Andrea Stein's AP Group.
 * For more information, see <a href="http://www.ai.mit.edu/projects/cs101">the
 * CS101 homepage</a> or email <las@ai.mit.edu>.
 *
 * Copyright (C) 1998 Massachusetts Institute of Technology.
 * Please do not redistribute without obtaining permission.
 */

package Spirograph;

import java.awt.*;
import java.util.*;

/**
 * This class keeps track of the coordinates of the Dot. It has a seperate
 * Thread to update the "state" of the two coordinates and to repaint
 * itself. It also handles the dot bouncing off either the walls of the
 * square or the circle. It also keeps track of all of the gravity points
 * and all of the places where the dot has been before and draw them in. <p>
 * 
 * Copyright (C) 1998 Massachusetts Institute of Technology.
 *
 * @author  Luis Sarmenta, lfgs@cag.lcs.mit.edu
 * @author  Henry Wong, henryw@mit.edu
 * @version  $Id: DotPanel.java,v 1.6 1998/07/24 16:58:43 tparnell Exp $
 *
 * @see Coord
 * @see Spirograph
 * @see AccelHandler
 */
public class DotPanel extends Canvas implements Runnable {
    
  // This vector holds all of the points representing the ball's
  // various positions.    
  private Vector v = new Vector(100);
    
  // All of the gravitational points that have been added
  private Vector grav = new Vector(5);
    
  // This image is used for the double buffering.
  private Image buf;

  // These fields represent the size of the Panel. They should be
  // changed using the setter methods whenever the window is resized.    
  private int width = Spirograph.WIDTH;
  private int height = Spirograph.HEIGHT;

  // The coords represent the ball's coordinates along the x and
  // y axis.
  private Coord x;
  private Coord y;

  // Whether or not the DotPanel is in circular mode.
  private boolean circMode = false;

  // The two focii of the ellipse when in circular mode
  private Point focusA = new Point(0,0);
  private Point focusB = new Point(0,0);

  // Bounce and Wrap Modes
  private boolean bounceOn = false;
  private boolean wrapOn = false;
	 
  // Mode corresponding to accelMode for AccelHandlers
  private int myMode = AccelHandler.POSMODE;
    
  public DotPanel (Coord x, Coord y) {
    this.x = x;
    this.y = y;
    this.setBackground (Color.white);
    this.setSize(width,height);
  }

  public void setMode( int mode )
  {
    this.myMode = mode;
    x.setMode( mode );
    y.setMode( mode );
  }

  public Dimension getPreferredSize() {
    return (new Dimension(Spirograph.WIDTH,Spirograph.HEIGHT));
  }

  public Dimension getMinimumSize() {
    return getPreferredSize();
  }
	 
  public double getMaxX()
  {
    return ( width/2 - Spirograph.BALLSIZE );
  }
  
  public double getMaxY()
  {
    return ( height/2 - Spirograph.BALLSIZE );
  }
  
  public void run() {
    // In order to make sure that the x and y axis go through the
    // same number of "timesteps" I use a seperate Thread to handle
    // painting and timestepping.
    
    // If you put the nextStep command in the AccelHandler threads
    // than you can't guarentee that the two timesteps will be
    // called with equal frequency.
    
    while (true) {
      double step;
      
      // I scale the timesteps so that there are more of them as
      // the dot get's nearer the edge. This way, the dot is
      // much more acccurate near the edge and it cuts down on
      // the flickering of the screen near the middle, where the
      // dot doesn't need to be as accurate
      
      if (circMode) {
	step = Spirograph.TIMESTEP*Math.abs(1
					    - (Math.pow(2*x.getPos()/height,2)
+ Math.pow(2*y.getPos()/width,2))/2);
      } else {
	step = 0.25;
      }
      
      // getDistVect creates a Vector containing the distance of
      // the dot from every gravitational point (see below)
      x.nextStep(getDistVect(true),step);
      y.nextStep(getDistVect(false),step);
      
      double xVel = x.getVel();
      double yVel = y.getVel();
      
      if (Math.sqrt(Math.pow(xVel,2) + Math.pow(yVel,2)) >
	  Spirograph.MAXVEL) {
	xVel = Spirograph.MAXVEL*xVel/(xVel+yVel);
	yVel = Spirograph.MAXVEL*yVel/(xVel+yVel);
	x.setVel(xVel);
	y.setVel(yVel);
      }
	    
      double xPos = x.getPos();
      double yPos = y.getPos();
	    	    	    
      if (circMode) {
	// This is a lot of math to figure out whether or not
	// the dot hit the wall and if it did, where it should
	// have gone and what it's new velocity should be.
		
	// from (x/a)^2 + (y/b)^2 = 1
	double a = width/2;
	double b = height/2;
	    
	if (!Util.inEllipse(xPos,yPos,a,b)) {

	  // Need to figure out where dot went out of circle.

	  double outXPos;
	  double outYPos;		    
		    
	  if (xVel == 0) {
	    // If this is true will get div 0 errors

	    outXPos = xPos;

	    if (yVel > 0) {
	      outYPos = b*Math.sqrt(1-Math.pow(xPos/a,2));
	    } else {
	      outYPos = -b*Math.sqrt(1-Math.pow(xPos/a,2));
	    }
	  } else {
			    
	    // Figure out "m" and "c" from (y = mx + c)
	    double m = yVel/xVel;
	    double c = yPos - (yVel*xPos/xVel);
			
	    // Figure out b^2-4ac...
	    double det = 4*Math.pow(a*b,2)*(Math.pow(b,2) +
					    Math.pow(a*m,2) - Math.pow(c,2));
			
		    
	    //Figure out +/-
			
	    if (xVel > 0) {
	      outXPos = (-2*m*c*Math.pow(a,2) +
			 Math.sqrt(det)) /
		(2*(Math.pow(b,2)+(Math.pow(m*a,2))));
	    } else {
	      outXPos = (-2*m*c*Math.pow(a,2) -
			 Math.sqrt(det)) /
		(2*(Math.pow(b,2)+(Math.pow(m*a,2))));
	    }
			
	    // now do same for y...
			
	    det = 4*Math.pow(a*m*b,2)*(Math.pow(a*m,2)+
				       Math.pow(b,2)-Math.pow(c,2));
			
	    if (yVel > 0) {
	      outYPos = (2*c*Math.pow(b,2) + Math.sqrt(det))/
		(2*(Math.pow(b,2)+Math.pow(m*a,2)));
	    } else {
	      outYPos = (2*c*Math.pow(b,2) - Math.sqrt(det))/
		(2*(Math.pow(b,2)+Math.pow(m*a,2)));
	    }
			
	  } // matches else from if (xVel == 0)

	  if (outXPos == 0) {
	    //Special case, refSlope = infinity

	    y.setVel(-y.getVel());
	    if (yPos >  0) {
	      y.setPos(height - yPos);
	    } else {
	      y.setPos(-height - yPos);
	    }

	  } else {
		    
	    double refSlope = -1*(outYPos*Math.pow(a,2)/
				  (-outXPos*Math.pow(b,2)));
	    double coef = (xVel + yVel * refSlope)/
	      (1 + Math.pow(refSlope,2));
		    
	    x.setVel(xVel- 2 * coef);
	    y.setVel(yVel- 2 * coef * refSlope);
			
	    coef = ((xPos - outXPos) + (yPos - outYPos) * refSlope)
	      /(1 + Math.pow(refSlope,2));
			
	    x.setPos(outXPos - 2 * coef);
	    y.setPos(outYPos - 2 * coef * refSlope);
 				
	  } // closes else from if (outX == 0)

	} // ends circMode out of bounds

      } else if ( bounceOn ) {
	// not circMode

	// If the ball moved past one of the boundaries,
	// calculate where it would have gone if it had bounced
	// and negate it's velocity.
		
	if (xPos > getMaxX() ) {
	  x.setPos(width - xPos - (Spirograph.BALLSIZE * 2));
	  x.setVel(-xVel);
	}
		
	if (xPos < -getMaxX() ) {
	  x.setPos(-width - xPos + (Spirograph.BALLSIZE * 2));
	  x.setVel(-xVel);
	}
		
	if (yPos > getMaxY() ) {
	  y.setPos(height - yPos - (Spirograph.BALLSIZE * 2));
	  y.setVel(-yVel);
	}
		
	if (yPos < -getMaxY() ) {
	  y.setPos(-height - yPos + (Spirograph.BALLSIZE * 2));
	  y.setVel(-yVel);
	}
      } else if (wrapOn) {
	if(xPos > getMaxX())
	  x.setPos(-getMaxX());
	else
	  if(xPos < -getMaxX())
	    x.setPos(getMaxX());
	if(yPos > getMaxY())
	  y.setPos(-getMaxY());
	else
	  if(yPos < -getMaxY())
	    y.setPos(getMaxY());
      } else {
	if(xPos > getMaxX())
	  x.setPos(getMaxX());
	else
	  if(xPos < -getMaxX())
	    x.setPos(-getMaxX());
	if(yPos > getMaxY())
	  y.setPos(getMaxY());
	else
	  if(yPos < -getMaxY())
	    y.setPos(-getMaxY());
      }

            
	    
      paintBuf();
      try {
	Thread.sleep((int)(100*step));
	//Thread.yield();
      } catch (InterruptedException e) {
	System.out.println ("Error sleeping.");
      }
    }
  }

  /** 
   * This is called by the AdvArg to see whether or not the ball is in
   * the circle. If the ball isn't inside the circle, AdvArg won't
   * turn on circle mode.
   */
  public boolean inEllipse() {
    return Util.inEllipse(x.getPos(), y.getPos(), width/2, height/2);
  }

  /** 
   * This method returns a vector of Points. The x coordinate of each
   * Point represents the distance of the point from the gravitational
   * dot along the appropriate axis, the y coordinate represents the
   * total distance of the dot from each point.
   *
   * @see Coord  
   */
  private Vector getDistVect(boolean xDim) {
    double xPos = x.getPos();
    double yPos = y.getPos();
    Vector temp = new Vector();
	
    for (Enumeration e = grav.elements(); e.hasMoreElements();) {
	    
      Point o = (Point)e.nextElement();

      if (xDim) {
	temp.addElement(new Point((int)(o.x-xPos),
				  (int)Util.dist(o.x,o.y,xPos,yPos)));
      } else {
	temp.addElement(new Point((int)(o.y-yPos),
				  (int)Util.dist(o.x,o.y,xPos,yPos)));
      }
    }

    return temp;
  }

  /**
   * This method paints an off-screen image to be used for the double
   * buffering.<p>
   *
   * It adds the balls current position to the vector of Points, then
   * reconstructs the balls path from the Points.
   */
  public void paintBuf() {
	
    Graphics g;

    if (buf == null) {
      buf = createImage(width, height);
    }
	
    try {
      g = buf.getGraphics();
    } catch (NullPointerException e) {
      return;
    }	

    g.clearRect(0,0,width,height);

    myMode = x.getMode();
    g.drawString("Pos: (" + x.getPos() + "," + y.getPos() + ")",5,15);
    if ( myMode != AccelHandler.POSMODE) 
      g.drawString("Vel: (" + x.getVel() + "," + y.getVel() + ")",5,30);
    if ( myMode == AccelHandler.ACCELMODE)
      g.drawString("Accel: ("+x.getAccel() +","+ y.getAccel() + ")",5,45);
	
    // Draw the ellipse
    if (circMode) {
      g.setColor(Color.black);
      g.drawOval(0,0,width,height);

      // Draw the focii
      g.drawOval(focusA.x+width/2-Spirograph.FOCUSSIZE/2,
		 height/2-focusA.y-Spirograph.FOCUSSIZE/2,
		 Spirograph.FOCUSSIZE, Spirograph.FOCUSSIZE);
      g.drawOval(focusB.x+width/2-Spirograph.FOCUSSIZE/2,
		 height/2-focusB.y-Spirograph.FOCUSSIZE/2,
		 Spirograph.FOCUSSIZE, Spirograph.FOCUSSIZE);
    }

    // Draw the gravity sources
    g.setColor(Color.blue);

    for (Enumeration e = grav.elements(); e.hasMoreElements();) {
	    
      Point o = (Point)e.nextElement();

      g.fillOval(o.x + width/2 - Spirograph.BALLSIZE/2,
		 height/2 - o.y - Spirograph.BALLSIZE/2,
		 Spirograph.BALLSIZE, Spirograph.BALLSIZE);
    }

    // Draw the dot's trail
    double oldX;
    double oldY;

    // Set up the beginning of the trail.
    if (v.isEmpty()) {
      oldX = x.getPos();
      oldY = y.getPos();
      v.addElement(new Point((int)x.getPos(),(int)y.getPos()) );  // add element now so call to v.lastElement later will work
    } else {
      oldX = ((Point)(v.elementAt(0))).x;
      oldY = ((Point)(v.elementAt(0))).y;
    }
	
    // Add the position of the dot to the vector of Points.
    Point newPoint = new Point((int)x.getPos(),(int)y.getPos());
    Point prevPoint = (Point)v.lastElement();  
	
    if ( ! (( Math.abs(newPoint.x-prevPoint.x) < 1.0 ) &&
	    ( Math.abs(newPoint.y-prevPoint.y) < 1.0 )) )
      v.addElement ( newPoint );
    // else no need to add a new point
	
    g.setColor (Color.red);

    // Step through the vector and draw the a line representing the
    // dot's path.
    for (Enumeration e = v.elements(); e.hasMoreElements();) {
	    
      Point o = (Point)e.nextElement();
      // if the difference between the old and new position
      // is almost as big as the width/height of the box,
      // then a wraparound probably happened, so don't draw a line.
      // otherwise, draw a line
      if( (Math.abs(o.x - oldX) < 2*getMaxX()-Spirograph.BALLSIZE)
	  && 
	  (Math.abs(o.y - oldY) < 2*getMaxY()-Spirograph.BALLSIZE)
	  ) {
	g.drawLine((int)(oldX+width/2), (int)(-oldY+height/2),
		   (int)(o.x+width/2), (int)(-o.y+height/2));
      }
	
      oldX = o.x;
      oldY = o.y;

    }
	
    g.setColor(Color.black);

    // Draw the oval
    g.fillOval ((int)(x.getPos()-Spirograph.BALLSIZE/2
		      +(width/2)),
		(int)(-y.getPos()-Spirograph.BALLSIZE/2
		      +(height/2)),
		Spirograph.BALLSIZE,Spirograph.BALLSIZE);	

    //  System.out.println ("NOW AT: " + x.getPos() + "," + y.getPos());

    repaint();
    g.dispose();
  } // end of paintbuf();
	
  public void paint (Graphics out) {
    // Draw the ball on screen. If buf hasn't been initialized yet,
    // initialized it.
	
    try {
      out.drawImage(buf,0,0,this);
    } catch (NullPointerException e) {
      paintBuf();
    }
	
  }

  /** Called by the DotFrame whenever the DotFrame is resized */
  public void setSize(int width, int height) {

    buf = null;
	
    this.width = width;
    this.height = height;
	
    x.setMaxPos( getMaxX() );
    y.setMaxPos( getMaxY() );

    // get new focii

    if (width < height) {
      focusA.x = 0;
      focusA.y = (int)Math.sqrt(Math.pow(height/2,2)-
				Math.pow(width/2,2));
      focusB.x = 0;
      focusB.y = -(int)Math.sqrt(Math.pow(height/2,2)-
				 Math.pow(width/2,2));
    } else {
      focusA.x = (int)Math.sqrt(Math.pow(width/2,2)-
				Math.pow(height/2,2));
      focusA.y = 0;
      focusB.x = -(int)Math.sqrt(Math.pow(width/2,2)-
				 Math.pow(height/2,2));
      focusB.y = 0;
    }

    if (circMode) {	
      System.out.println ("Focus A at: "+focusA.x+":"+focusA.y);
      System.out.println ("Focus B at: "+focusB.x+":"+focusB.y);
    }
	
    paintBuf();
    super.setSize(width,height);
  }

  // Methods to set and get various things.
  public void flushLines() {
    v.removeAllElements();
    paintBuf();
  }

  public void flushGrav() {
    grav.removeAllElements();
    paintBuf();
  }
    
  public void addGrav(int x, int y) {
    grav.addElement(new Point(x,y));
  }

  public void setCirc(boolean circMode) {
    this.circMode = circMode;
    paintBuf();
  }

  public boolean getCirc() {
    return circMode;
  }
    
  public int curHeight() {
    return height;
  }

  public int curWidth() {
    return width;
  }

  public boolean getBounce()
  {
    return bounceOn;
  }

  public void setBounce(boolean bounceOn)
  {
    this.bounceOn = bounceOn;
  }

  public boolean getWrap()
  {
    return wrapOn;
  }

  public void setWrap(boolean wrapOn)
  {
    this.wrapOn = wrapOn;
  }
}


/*
 * $Log: DotPanel.java,v $
 * Revision 1.6  1998/07/24 16:58:43  tparnell
 * Placate new javadoc behavior
 *
 * Revision 1.5  1998/07/24 15:51:36  tparnell
 * Added javadoc.  Removed some deprecated code.  Fixed a few minor bugs.
 *
 * Revision 1.4  1998/07/23 15:09:29  natashao
 * Added javadoc and a few comments.
 * */




